#ifndef _CONSTANTS_H_
#define _CONSTANTS_H_
#if (defined(_MSC_VER) && (_MSC_VER > 1000))
#pragma once
#endif /* (defined(_MSC_VER) && (_MSC_VER > 1000)) */
#if defined(_WIN32) && defined(_MANAGED)
#pragma managed(push, off)
#endif /* defined(_WIN32) && defined(_MANAGED) */

#ifdef _WIN32
#define OGL4COREPLUGIN_API __declspec(dllexport)
#else
#define OGL4COREPLUGIN_API
#endif

#define OGLC_RELEASE                0
/*! @brief The key or button was pressed.
 *  @ingroup input
 */
#define OGLC_PRESS                  1
/*! @brief The key was held down until it repeated.
 *  @ingroup input
 */
#define OGLC_REPEAT                 2
/*! @} */

/*! @defgroup keys Keyboard keys
 *
 * These key codes are inspired by the *USB HID Usage Tables v1.12* (p. 53-60),
 * but re-arranged to map to 7-bit ASCII for printable keys (function keys are
 * put in the 256+ range).
 *
 * The naming of the key codes follow these rules:
 *  - The US keyboard layout is used
 *  - Names of printable alpha-numeric characters are used (e.g. "A", "R",
 *    "3", etc.)
 *  - For non-alphanumeric characters, Unicode:ish names are used (e.g.
 *    "COMMA", "LEFT_SQUARE_BRACKET", etc.). Note that some names do not
 *    correspond to the Unicode standard (usually for brevity)
 *  - Keys that lack a clear US mapping are named "WORLD_x"
 *  - For non-printable keys, custom names are used (e.g. "F4",
 *    "BACKSPACE", etc.)
 *
 *  @ingroup input
 *  @{
 */

/* The unknown key */
#define OGLC_KEY_UNKNOWN            -1

/* Printable keys */
#define OGLC_KEY_SPACE              32
#define OGLC_KEY_APOSTROPHE         39  /* ' */
#define OGLC_KEY_COMMA              44  /* , */
#define OGLC_KEY_MINUS              45  /* - */
#define OGLC_KEY_PERIOD             46  /* . */
#define OGLC_KEY_SLASH              47  /* / */
#define OGLC_KEY_0                  48
#define OGLC_KEY_1                  49
#define OGLC_KEY_2                  50
#define OGLC_KEY_3                  51
#define OGLC_KEY_4                  52
#define OGLC_KEY_5                  53
#define OGLC_KEY_6                  54
#define OGLC_KEY_7                  55
#define OGLC_KEY_8                  56
#define OGLC_KEY_9                  57
#define OGLC_KEY_SEMICOLON          59  /* ; */
#define OGLC_KEY_EQUAL              61  /* = */
#define OGLC_KEY_A                  65
#define OGLC_KEY_B                  66
#define OGLC_KEY_C                  67
#define OGLC_KEY_D                  68
#define OGLC_KEY_E                  69
#define OGLC_KEY_F                  70
#define OGLC_KEY_G                  71
#define OGLC_KEY_H                  72
#define OGLC_KEY_I                  73
#define OGLC_KEY_J                  74
#define OGLC_KEY_K                  75
#define OGLC_KEY_L                  76
#define OGLC_KEY_M                  77
#define OGLC_KEY_N                  78
#define OGLC_KEY_O                  79
#define OGLC_KEY_P                  80
#define OGLC_KEY_Q                  81
#define OGLC_KEY_R                  82
#define OGLC_KEY_S                  83
#define OGLC_KEY_T                  84
#define OGLC_KEY_U                  85
#define OGLC_KEY_V                  86
#define OGLC_KEY_W                  87
#define OGLC_KEY_X                  88
#define OGLC_KEY_Y                  89
#define OGLC_KEY_Z                  90
#define OGLC_KEY_LEFT_BRACKET       91  /* [ */
#define OGLC_KEY_BACKSLASH          92  /* \ */
#define OGLC_KEY_RIGHT_BRACKET      93  /* ] */
#define OGLC_KEY_GRAVE_ACCENT       96  /* ` */
#define OGLC_KEY_WORLD_1            161 /* non-US #1 */
#define OGLC_KEY_WORLD_2            162 /* non-US #2 */

/* Function keys */
#define OGLC_KEY_ESCAPE             256
#define OGLC_KEY_ENTER              257
#define OGLC_KEY_TAB                258
#define OGLC_KEY_BACKSPACE          259
#define OGLC_KEY_INSERT             260
#define OGLC_KEY_DELETE             261
#define OGLC_KEY_RIGHT              262
#define OGLC_KEY_LEFT               263
#define OGLC_KEY_DOWN               264
#define OGLC_KEY_UP                 265
#define OGLC_KEY_PAGE_UP            266
#define OGLC_KEY_PAGE_DOWN          267
#define OGLC_KEY_HOME               268
#define OGLC_KEY_END                269
#define OGLC_KEY_CAPS_LOCK          280
#define OGLC_KEY_SCROLL_LOCK        281
#define OGLC_KEY_NUM_LOCK           282
#define OGLC_KEY_PRINT_SCREEN       283
#define OGLC_KEY_PAUSE              284
#define OGLC_KEY_F1                 290
#define OGLC_KEY_F2                 291
#define OGLC_KEY_F3                 292
#define OGLC_KEY_F4                 293
#define OGLC_KEY_F5                 294
#define OGLC_KEY_F6                 295
#define OGLC_KEY_F7                 296
#define OGLC_KEY_F8                 297
#define OGLC_KEY_F9                 298
#define OGLC_KEY_F10                299
#define OGLC_KEY_F11                300
#define OGLC_KEY_F12                301
#define OGLC_KEY_F13                302
#define OGLC_KEY_F14                303
#define OGLC_KEY_F15                304
#define OGLC_KEY_F16                305
#define OGLC_KEY_F17                306
#define OGLC_KEY_F18                307
#define OGLC_KEY_F19                308
#define OGLC_KEY_F20                309
#define OGLC_KEY_F21                310
#define OGLC_KEY_F22                311
#define OGLC_KEY_F23                312
#define OGLC_KEY_F24                313
#define OGLC_KEY_F25                314
#define OGLC_KEY_KP_0               320
#define OGLC_KEY_KP_1               321
#define OGLC_KEY_KP_2               322
#define OGLC_KEY_KP_3               323
#define OGLC_KEY_KP_4               324
#define OGLC_KEY_KP_5               325
#define OGLC_KEY_KP_6               326
#define OGLC_KEY_KP_7               327
#define OGLC_KEY_KP_8               328
#define OGLC_KEY_KP_9               329
#define OGLC_KEY_KP_DECIMAL         330
#define OGLC_KEY_KP_DIVIDE          331
#define OGLC_KEY_KP_MULTIPLY        332
#define OGLC_KEY_KP_SUBTRACT        333
#define OGLC_KEY_KP_ADD             334
#define OGLC_KEY_KP_ENTER           335
#define OGLC_KEY_KP_EQUAL           336
#define OGLC_KEY_LEFT_SHIFT         340
#define OGLC_KEY_LEFT_CONTROL       341
#define OGLC_KEY_LEFT_ALT           342
#define OGLC_KEY_LEFT_SUPER         343
#define OGLC_KEY_RIGHT_SHIFT        344
#define OGLC_KEY_RIGHT_CONTROL      345
#define OGLC_KEY_RIGHT_ALT          346
#define OGLC_KEY_RIGHT_SUPER        347
#define OGLC_KEY_MENU               348
#define OGLC_KEY_LAST               OGLC_KEY_MENU

/*! @} */

/*! @defgroup mods Modifier key flags
 *  @ingroup input
 *  @{ */

/*! @brief If this bit is set one or more Shift keys were held down.
 */
#define OGLC_MOD_SHIFT           0x0001
/*! @brief If this bit is set one or more Control keys were held down.
 */
#define OGLC_MOD_CONTROL         0x0002
/*! @brief If this bit is set one or more Alt keys were held down.
 */
#define OGLC_MOD_ALT             0x0004
/*! @brief If this bit is set one or more Super keys were held down.
 */
#define OGLC_MOD_SUPER           0x0008

/*! @} */

/*! @defgroup buttons Mouse buttons
 *  @ingroup input
 *  @{ */
#define OGLC_MOUSE_BUTTON_1         0
#define OGLC_MOUSE_BUTTON_2         1
#define OGLC_MOUSE_BUTTON_3         2
#define OGLC_MOUSE_BUTTON_4         3
#define OGLC_MOUSE_BUTTON_5         4
#define OGLC_MOUSE_BUTTON_6         5
#define OGLC_MOUSE_BUTTON_7         6
#define OGLC_MOUSE_BUTTON_8         7
#define OGLC_MOUSE_BUTTON_LAST      OGLC_MOUSE_BUTTON_8
#define OGLC_MOUSE_BUTTON_LEFT      OGLC_MOUSE_BUTTON_1
#define OGLC_MOUSE_BUTTON_RIGHT     OGLC_MOUSE_BUTTON_2
#define OGLC_MOUSE_BUTTON_MIDDLE    OGLC_MOUSE_BUTTON_3
/*! @} */

/*! @defgroup joysticks Joysticks
 *  @ingroup input
 *  @{ */
//#define OGLC_JOYSTICK_1             0
//#define OGLC_JOYSTICK_2             1
//#define OGLC_JOYSTICK_3             2
//#define OGLC_JOYSTICK_4             3
//#define OGLC_JOYSTICK_5             4
//#define OGLC_JOYSTICK_6             5
//#define OGLC_JOYSTICK_7             6
//#define OGLC_JOYSTICK_8             7
//#define OGLC_JOYSTICK_9             8
//#define OGLC_JOYSTICK_10            9
//#define OGLC_JOYSTICK_11            10
//#define OGLC_JOYSTICK_12            11
//#define OGLC_JOYSTICK_13            12
//#define OGLC_JOYSTICK_14            13
//#define OGLC_JOYSTICK_15            14
//#define OGLC_JOYSTICK_16            15
//#define OGLC_JOYSTICK_LAST          OGLC_JOYSTICK_16
/*! @} */

#if defined(_WIN32) && defined(_MANAGED)
#pragma managed(pop)
#endif /* defined(_WIN32) && defined(_MANAGED) */
#endif /* _CONSTANTS_H_ */
